@@ -13,20 +13,24 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
+import copy
+
 from robot.api import logger
-from robot.utils import (is_dict_like, is_string, is_truthy, plural_or_not,
-                         seq2str, seq2str2, type_name, unic, Matcher)
+from robot.utils import (is_dict_like, is_list_like, is_number, is_string, is_truthy,
+                         Matcher, plural_or_not, seq2str, seq2str2, type_name)
 from robot.utils.asserts import assert_equal
 from robot.version import get_version
 
 
-class NotSet(object):
+class NotSet:
     def __repr__(self):
         return ""
+
+
 NOT_SET = NotSet()
 
 
-class _List(object):
+class _List:
 
     def convert_to_list(self, item):
         """Converts the given ``item`` to a Python ``list`` type.
@@ -46,6 +50,7 @@ class _List(object):
         | ${L1} = ['a', 'xxx']
         | ${L2} = ['a', 'b', 'x', 'y', 'z']
         """
+        self._validate_list(list_)
         for value in values:
             list_.append(value)
 
@@ -70,6 +75,7 @@ class _List(object):
         | ${L1} = ['xxx', 'a']
         | ${L2} = ['a', 'xxx', 'b']
         """
+        self._validate_list(list_)
         list_.insert(self._index_to_int(index), value)
 
     def combine_lists(self, *lists):
@@ -78,13 +84,14 @@ class _List(object):
         The given lists are not altered by this keyword.
 
         Example:
-        | ${x} = | Combine List | ${L1} | ${L2} |       |
-        | ${y} = | Combine List | ${L1} | ${L2} | ${L1} |
+        | ${x} = | Combine Lists | ${L1} | ${L2} |       |
+        | ${y} = | Combine Lists | ${L1} | ${L2} | ${L1} |
         =>
         | ${x} = ['a', 'a', 'b']
         | ${y} = ['a', 'a', 'b', 'a']
         | ${L1} and ${L2} are not changed.
         """
+        self._validate_lists(*lists)
         ret = []
         for item in lists:
             ret.extend(item)
@@ -105,6 +112,7 @@ class _List(object):
         =>
         | ${L3} = ['a', 'xxx', 'yyy']
         """
+        self._validate_list(list_)
         try:
             list_[self._index_to_int(index)] = value
         except IndexError:
@@ -120,6 +128,7 @@ class _List(object):
         =>
         | ${L4} = ['b', 'd']
         """
+        self._validate_list(list_)
         for value in values:
             while value in list_:
                 list_.remove(value)
@@ -139,6 +148,7 @@ class _List(object):
         | ${x} = 'a'
         | ${L2} = ['b']
         """
+        self._validate_list(list_)
         try:
             return list_.pop(self._index_to_int(index))
         except IndexError:
@@ -151,9 +161,8 @@ class _List(object):
         list so that one item can appear only once. Order of the items in
         the new list is the same as in the original except for missing
         duplicates. Number of the removed duplicates is logged.
-
-        New in Robot Framework 2.7.5.
         """
+        self._validate_list(list_)
         ret = []
         for item in list_:
             if item not in ret:
@@ -181,6 +190,7 @@ class _List(object):
         | ${y} = 'd'
         | ${L5} is not changed
         """
+        self._validate_list(list_)
         try:
             return list_[self._index_to_int(index)]
         except IndexError:
@@ -201,15 +211,16 @@ class _List(object):
         the largest (or smallest) available index.
 
         Examples (incl. Python equivalents in comments):
-        | ${x} = | Get Slice From List | ${L5} | 2 | 4  | # L5[2:4]    |
-        | ${y} = | Get Slice From List | ${L5} | 1 |    | # L5[1:None] |
-        | ${z} = | Get Slice From List | ${L5} |   | -2 | # L5[0:-2]   |
+        | ${x} = | Get Slice From List | ${L5} | 2      | 4 | # L5[2:4]    |
+        | ${y} = | Get Slice From List | ${L5} | 1      |   | # L5[1:None] |
+        | ${z} = | Get Slice From List | ${L5} | end=-2 |   | # L5[0:-2]   |
         =>
         | ${x} = ['c', 'd']
         | ${y} = ['b', 'c', 'd', 'e']
         | ${z} = ['a', 'b', 'c']
         | ${L5} is not changed
         """
+        self._validate_list(list_)
         start = self._index_to_int(start, True)
         if end is not None:
             end = self._index_to_int(end)
@@ -228,6 +239,7 @@ class _List(object):
         | ${x} = 1
         | ${L3} is not changed
         """
+        self._validate_list(list_)
         return self.get_slice_from_list(list_, start, end).count(value)
 
     def get_index_from_list(self, list_, value, start=0, end=None):
@@ -244,6 +256,7 @@ class _List(object):
         | ${x} = 3
         | ${L5} is not changed
         """
+        self._validate_list(list_)
         if start == '':
             start = 0
         list_ = self.get_slice_from_list(list_, start, end)
@@ -252,11 +265,17 @@ class _List(object):
         except ValueError:
             return -1
 
-    def copy_list(self, list_):
+    def copy_list(self, list_, deepcopy=False):
         """Returns a copy of the given list.
 
+        If the optional ``deepcopy`` is given a true value, the returned
+        list is a deep copy. New option in Robot Framework 3.1.2.
+
         The given list is never altered by this keyword.
         """
+        self._validate_list(list_)
+        if deepcopy:
+            return copy.deepcopy(list_)
         return list_[:]
 
     def reverse_list(self, list_):
@@ -269,38 +288,37 @@ class _List(object):
         =>
         | ${L3} = ['c', 'b', 'a']
         """
+        self._validate_list(list_)
         list_.reverse()
 
     def sort_list(self, list_):
         """Sorts the given list in place.
 
-        The strings are sorted alphabetically and the numbers numerically.
+        Sorting fails if items in the list are not comparable with each others.
+        On Python 2 most objects are comparable, but on Python 3 comparing,
+        for example, strings with numbers is not possible.
 
         Note that the given list is changed and nothing is returned. Use
         `Copy List` first, if you need to keep also the original order.
-
-        ${L} = [2,1,'a','c','b']
-        | Sort List | ${L} |
-        =>
-        | ${L} = [1, 2, 'a', 'b', 'c']
         """
+        self._validate_list(list_)
         list_.sort()
 
     def list_should_contain_value(self, list_, value, msg=None):
         """Fails if the ``value`` is not found from ``list``.
 
-        If the keyword fails, the default error messages is ``<list> does
-        not contain value '<value>'``. A custom message can be given using
-        the ``msg`` argument.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_list(list_)
         default = "%s does not contain value '%s'." % (seq2str2(list_), value)
         _verify_condition(value in list_, default, msg)
 
     def list_should_not_contain_value(self, list_, value, msg=None):
         """Fails if the ``value`` is found from ``list``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_list(list_)
         default = "%s contains value '%s'." % (seq2str2(list_), value)
         _verify_condition(value not in list_, default, msg)
 
@@ -315,6 +333,7 @@ class _List(object):
         This keyword works with all iterables that can be converted to a list.
         The original iterable is never altered.
         """
+        self._validate_list(list_)
         if not isinstance(list_, list):
             list_ = list(list_)
         dupes = []
@@ -329,7 +348,7 @@ class _List(object):
                                  '%s found multiple times.' % seq2str(dupes))
 
     def lists_should_be_equal(self, list1, list2, msg=None, values=True,
-                              names=None):
+                              names=None, ignore_order=False):
         """Fails if given lists are unequal.
 
         The keyword first verifies that the lists have equal lengths, and then
@@ -338,7 +357,6 @@ class _List(object):
         Abc``. The types of the lists do not need to be the same. For example,
         Python tuple and list with same content are considered equal.
 
-
         The error message can be configured using ``msg`` and ``values``
         arguments:
         - If ``msg`` is not given, the default error message is used.
@@ -348,12 +366,12 @@ class _List(object):
         - If ``msg`` is given and ``values``  is not given a true value,
           the error message is just the given ``msg``.
 
-        Optional ``names`` argument can be used for naming the indices shown in
-        the default error message. It can either be a list of names matching
-        the indices in the lists or a dictionary where keys are indices that
-        need to be named. It is not necessary to name all of the indices.  When
-        using a dictionary, keys can be either integers or strings that can be
-        converted to integers.
+        The optional ``names`` argument can be used for naming the indices
+        shown in the default error message. It can either be a list of names
+        matching the indices in the lists or a dictionary where keys are
+        indices that need to be named. It is not necessary to name all of
+        the indices.  When using a dictionary, keys can be either integers
+        or strings that can be converted to integers.
 
         Examples:
         | ${names} = | Create List | First Name | Family Name | Email |
@@ -364,12 +382,25 @@ class _List(object):
         If the items in index 2 would differ in the above examples, the error
         message would contain a row like ``Index 2 (email): name@foo.com !=
         name@bar.com``.
+
+        The optional ``ignore_order`` argument can be used to ignore the order
+        of the elements in the lists. Using it requires items to be sortable.
+        This is new in Robot Framework 3.2.
+
+        Example:
+        | ${list1} = | Create List | apple | cherry | banana |
+        | ${list2} = | Create List | cherry | banana | apple |
+        | Lists Should Be Equal | ${list1} | ${list2} | ignore_order=True |
         """
+        self._validate_lists(list1, list2)
         len1 = len(list1)
         len2 = len(list2)
         default = 'Lengths are different: %d != %d' % (len1, len2)
         _verify_condition(len1 == len2, default, msg, values)
         names = self._get_list_index_name_mapping(names, len1)
+        if ignore_order:
+            list1 = sorted(list1)
+            list2 = sorted(list2)
         diffs = list(self._yield_list_diffs(list1, list2, names))
         default = 'Lists are different:\n' + '\n'.join(diffs)
         _verify_condition(diffs == [], default, msg, values)
@@ -387,7 +418,7 @@ class _List(object):
             try:
                 assert_equal(item1, item2, msg='Index %d%s' % (index, name))
             except AssertionError as err:
-                yield unic(err)
+                yield str(err)
 
     def list_should_contain_sub_list(self, list1, list2, msg=None, values=True):
         """Fails if not all of the elements in ``list2`` are found in ``list1``.
@@ -398,7 +429,8 @@ class _List(object):
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
         """
-        diffs = ', '.join(unic(item) for item in list2 if item not in list1)
+        self._validate_lists(list1, list2)
+        diffs = ', '.join(str(item) for item in list2 if item not in list1)
         default = 'Following values were not found from first list: ' + diffs
         _verify_condition(not diffs, default, msg, values)
 
@@ -410,13 +442,14 @@ class _List(object):
         If you only want to the length, use keyword `Get Length` from
         the BuiltIn library.
         """
+        self._validate_list(list_)
         logger.write('\n'.join(self._log_list(list_)), level)
 
     def _log_list(self, list_):
         if not list_:
             yield 'List is empty.'
         elif len(list_) == 1:
-            yield 'List has one item:\n%s' % list_[0]
+            yield 'List has one item:\n%s' % (list_[0],)
         else:
             yield 'List length is %d and it contains following items:' % len(list_)
             for index, item in enumerate(list_):
@@ -434,17 +467,27 @@ class _List(object):
         raise IndexError('Given index %s is out of the range 0-%d.'
                          % (index, len(list_)-1))
 
+    def _validate_list(self, list_, position=1):
+        if not is_list_like(list_):
+            raise TypeError("Expected argument %d to be a list or list-like, "
+                            "got %s instead." % (position, type_name(list_)))
 
-class _Dictionary(object):
+    def _validate_lists(self, *lists):
+        for index, item in enumerate(lists, start=1):
+            self._validate_list(item, index)
+
+
+class _Dictionary:
 
     def convert_to_dictionary(self, item):
         """Converts the given ``item`` to a Python ``dict`` type.
 
-        Mainly useful for converting other mappings to dictionaries. Use
-        `Create Dictionary` from the BuiltIn library for constructing new
-        dictionaries.
+        Mainly useful for converting other mappings to normal dictionaries.
+        This includes converting Robot Framework's own ``DotDict`` instances
+        that it uses if variables are created using the ``&{var}`` syntax.
 
-        New in Robot Framework 2.9.
+        Use `Create Dictionary` from the BuiltIn library for constructing new
+        dictionaries.
         """
         return dict(item)
 
@@ -458,9 +501,6 @@ class _Dictionary(object):
         =>
         | ${D1} = {'a': 1, 'key': 'value', 'second': 2}
 
-        Starting from Robot Framework 2.8.1, items can also be given as kwargs
-        using ``key=value`` syntax:
-
         | Set To Dictionary | ${D1} | key=value | second=${2} |
 
         The latter syntax is typically more convenient to use, but it has
@@ -468,6 +508,7 @@ class _Dictionary(object):
 
         If given keys already exist in the dictionary, their values are updated.
         """
+        self._validate_dictionary(dictionary)
         if len(key_value_pairs) % 2 != 0:
             raise ValueError("Adding data to a dictionary failed. There "
                              "should be even number of key-value-pairs.")
@@ -487,12 +528,13 @@ class _Dictionary(object):
         =>
         | ${D3} = {'a': 1, 'c': 3}
         """
+        self._validate_dictionary(dictionary)
         for key in keys:
             if key in dictionary:
                 value = dictionary.pop(key)
                 logger.info("Removed item with key '%s' and value '%s'." % (key, value))
             else:
-                logger.info("Key '%s' not found." % key)
+                logger.info("Key '%s' not found." % (key,))
 
     def pop_from_dictionary(self, dictionary, key, default=NOT_SET):
         """Pops the given ``key`` from the ``dictionary`` and returns its value.
@@ -506,9 +548,8 @@ class _Dictionary(object):
         =>
         | ${val} = 2
         | ${D3} = {'a': 1, 'c': 3}
-
-        New in Robot Framework 2.9.2.
         """
+        self._validate_dictionary(dictionary)
         if default is NOT_SET:
             self.dictionary_should_contain_key(dictionary, key)
             return dictionary.pop(key)
@@ -525,62 +566,110 @@ class _Dictionary(object):
         =>
         | ${D5} = {'b': 2, 'd': 4}
         """
+        self._validate_dictionary(dictionary)
         remove_keys = [k for k in dictionary if k not in keys]
         self.remove_from_dictionary(dictionary, *remove_keys)
 
-    def copy_dictionary(self, dictionary):
+    def copy_dictionary(self, dictionary, deepcopy=False):
         """Returns a copy of the given dictionary.
 
+        The ``deepcopy`` argument controls should the returned dictionary be
+        a [https://docs.python.org/library/copy.html|shallow or deep copy].
+        By default returns a shallow copy, but that can be changed by giving
+        ``deepcopy`` a true value (see `Boolean arguments`). This is a new
+        option in Robot Framework 3.1.2. Earlier versions always returned
+        shallow copies.
+
         The given dictionary is never altered by this keyword.
         """
+        self._validate_dictionary(dictionary)
+        if deepcopy:
+            return copy.deepcopy(dictionary)
         return dictionary.copy()
 
-    def get_dictionary_keys(self, dictionary):
-        """Returns keys of the given ``dictionary``.
+    def get_dictionary_keys(self, dictionary, sort_keys=True):
+        """Returns keys of the given ``dictionary`` as a list.
 
-        If keys are sortable, they are returned in sorted order. The given
-        ``dictionary`` is never altered by this keyword.
+        By default keys are returned in sorted order (assuming they are
+        sortable), but they can be returned in the original order by giving
+        ``sort_keys``  a false value (see `Boolean arguments`). Notice that
+        with Python 3.5 and earlier dictionary order is undefined unless using
+        ordered dictionaries.
+
+        The given ``dictionary`` is never altered by this keyword.
 
         Example:
-        | ${keys} = | Get Dictionary Keys | ${D3} |
+        | ${sorted} =   | Get Dictionary Keys | ${D3} |
+        | ${unsorted} = | Get Dictionary Keys | ${D3} | sort_keys=False |
         =>
-        | ${keys} = ['a', 'b', 'c']
+        | ${sorted} = ['a', 'b', 'c']
+        | ${unsorted} = ['b', 'a', 'c']   # Order depends on Python version.
+
+        ``sort_keys`` is a new option in Robot Framework 3.1.2. Earlier keys
+        were always sorted.
         """
-        # TODO: Possibility to disable sorting. Can be handy with OrderedDicts.
+        self._validate_dictionary(dictionary)
         keys = dictionary.keys()
-        try:
-            return sorted(keys)
-        except TypeError:
-            return list(keys)
+        if sort_keys:
+            try:
+                return sorted(keys)
+            except TypeError:
+                pass
+        return list(keys)
 
-    def get_dictionary_values(self, dictionary):
-        """Returns values of the given dictionary.
+    def get_dictionary_values(self, dictionary, sort_keys=True):
+        """Returns values of the given ``dictionary`` as a list.
 
-        Values are returned sorted according to keys. The given dictionary is
-        never altered by this keyword.
+        Uses `Get Dictionary Keys` to get keys and then returns corresponding
+        values. By default keys are sorted and values returned in that order,
+        but this can be changed by giving ``sort_keys`` a false value (see
+        `Boolean arguments`). Notice that with Python 3.5 and earlier
+        dictionary order is undefined unless using ordered dictionaries.
+
+        The given ``dictionary`` is never altered by this keyword.
 
         Example:
-        | ${values} = | Get Dictionary Values | ${D3} |
+        | ${sorted} =   | Get Dictionary Values | ${D3} |
+        | ${unsorted} = | Get Dictionary Values | ${D3} | sort_keys=False |
         =>
-        | ${values} = [1, 2, 3]
+        | ${sorted} = [1, 2, 3]
+        | ${unsorted} = [2, 1, 3]    # Order depends on Python version.
+
+        ``sort_keys`` is a new option in Robot Framework 3.1.2. Earlier values
+        were always sorted based on keys.
         """
-        return [dictionary[k] for k in self.get_dictionary_keys(dictionary)]
+        self._validate_dictionary(dictionary)
+        keys = self.get_dictionary_keys(dictionary, sort_keys=sort_keys)
+        return [dictionary[k] for k in keys]
 
-    def get_dictionary_items(self, dictionary):
-        """Returns items of the given ``dictionary``.
+    def get_dictionary_items(self, dictionary, sort_keys=True):
+        """Returns items of the given ``dictionary`` as a list.
 
-        Items are returned sorted by keys. The given ``dictionary`` is not
-        altered by this keyword.
+        Uses `Get Dictionary Keys` to get keys and then returns corresponding
+        items. By default keys are sorted and items returned in that order,
+        but this can be changed by giving ``sort_keys`` a false value (see
+        `Boolean arguments`). Notice that with Python 3.5 and earlier
+        dictionary order is undefined unless using ordered dictionaries.
+
+        Items are returned as a flat list so that first item is a key,
+        second item is a corresponding value, third item is the second key,
+        and so on.
+
+        The given ``dictionary`` is never altered by this keyword.
 
         Example:
-        | ${items} = | Get Dictionary Items | ${D3} |
+        | ${sorted} =   | Get Dictionary Items | ${D3} |
+        | ${unsorted} = | Get Dictionary Items | ${D3} | sort_keys=False |
         =>
-        | ${items} = ['a', 1, 'b', 2, 'c', 3]
+        | ${sorted} = ['a', 1, 'b', 2, 'c', 3]
+        | ${unsorted} = ['b', 2, 'a', 1, 'c', 3]    # Order depends on Python version.
+
+        ``sort_keys`` is a new option in Robot Framework 3.1.2. Earlier items
+        were always sorted based on keys.
         """
-        ret = []
-        for key in self.get_dictionary_keys(dictionary):
-            ret.extend((key, dictionary[key]))
-        return ret
+        self._validate_dictionary(dictionary)
+        keys = self.get_dictionary_keys(dictionary, sort_keys=sort_keys)
+        return [i for key in keys for i in (key, dictionary[key])]
 
     def get_from_dictionary(self, dictionary, key):
         """Returns a value from the given ``dictionary`` based on the given ``key``.
@@ -595,62 +684,59 @@ class _Dictionary(object):
         =>
         | ${value} = 2
         """
+        self._validate_dictionary(dictionary)
         try:
             return dictionary[key]
         except KeyError:
-            raise RuntimeError("Dictionary does not contain key '%s'." % key)
+            raise RuntimeError("Dictionary does not contain key '%s'." % (key,))
 
     def dictionary_should_contain_key(self, dictionary, key, msg=None):
         """Fails if ``key`` is not found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
-        default = "Dictionary does not contain key '%s'." % key
+        self._validate_dictionary(dictionary)
+        default = "Dictionary does not contain key '%s'." % (key,)
         _verify_condition(key in dictionary, default, msg)
 
     def dictionary_should_not_contain_key(self, dictionary, key, msg=None):
         """Fails if ``key`` is found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
-        default = "Dictionary contains key '%s'." % key
+        self._validate_dictionary(dictionary)
+        default = "Dictionary contains key '%s'." % (key,)
         _verify_condition(key not in dictionary, default, msg)
 
     def dictionary_should_contain_item(self, dictionary, key, value, msg=None):
-        """An item of ``key``/``value`` must be found in a `dictionary`.
+        """An item of ``key`` / ``value`` must be found in a ``dictionary``.
 
         Value is converted to unicode for comparison.
 
-        See `Lists Should Be Equal` for an explanation of ``msg``.
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
+        self._validate_dictionary(dictionary)
         self.dictionary_should_contain_key(dictionary, key, msg)
-        actual, expected = unic(dictionary[key]), unic(value)
+        actual, expected = str(dictionary[key]), str(value)
         default = "Value of dictionary key '%s' does not match: %s != %s" % (key, actual, expected)
         _verify_condition(actual == expected, default, msg)
 
     def dictionary_should_contain_value(self, dictionary, value, msg=None):
         """Fails if ``value`` is not found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
-        default = "Dictionary does not contain value '%s'." % value
+        self._validate_dictionary(dictionary)
+        default = "Dictionary does not contain value '%s'." % (value,)
         _verify_condition(value in dictionary.values(), default, msg)
 
     def dictionary_should_not_contain_value(self, dictionary, value, msg=None):
         """Fails if ``value`` is found from ``dictionary``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
-        The given dictionary is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
         """
-        default = "Dictionary contains value '%s'." % value
+        self._validate_dictionary(dictionary)
+        default = "Dictionary contains value '%s'." % (value,)
         _verify_condition(not value in dictionary.values(), default, msg)
 
     def dictionaries_should_be_equal(self, dict1, dict2, msg=None, values=True):
@@ -663,9 +749,9 @@ class _Dictionary(object):
 
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
-
-        The given dictionaries are never altered by this keyword.
         """
+        self._validate_dictionary(dict1)
+        self._validate_dictionary(dict2, 2)
         keys = self._keys_should_be_equal(dict1, dict2, msg, values)
         self._key_values_should_be_equal(keys, dict1, dict2, msg, values)
 
@@ -675,11 +761,11 @@ class _Dictionary(object):
 
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
-
-        The given dictionaries are never altered by this keyword.
         """
+        self._validate_dictionary(dict1)
+        self._validate_dictionary(dict2, 2)
         keys = self.get_dictionary_keys(dict2)
-        diffs = [unic(k) for k in keys if k not in dict1]
+        diffs = [str(k) for k in keys if k not in dict1]
         default = "Following keys missing from first dictionary: %s" \
                   % ', '.join(diffs)
         _verify_condition(not diffs, default, msg, values)
@@ -693,6 +779,7 @@ class _Dictionary(object):
         If you only want to log the size, use keyword `Get Length` from
         the BuiltIn library.
         """
+        self._validate_dictionary(dictionary)
         logger.write('\n'.join(self._log_dictionary(dictionary)), level)
 
     def _log_dictionary(self, dictionary):
@@ -708,8 +795,8 @@ class _Dictionary(object):
     def _keys_should_be_equal(self, dict1, dict2, msg, values):
         keys1 = self.get_dictionary_keys(dict1)
         keys2 = self.get_dictionary_keys(dict2)
-        miss1 = [unic(k) for k in keys2 if k not in dict1]
-        miss2 = [unic(k) for k in keys1 if k not in dict2]
+        miss1 = [str(k) for k in keys2 if k not in dict1]
+        miss2 = [str(k) for k in keys1 if k not in dict2]
         error = []
         if miss1:
             error += ['Following keys missing from first dictionary: %s'
@@ -730,11 +817,16 @@ class _Dictionary(object):
             try:
                 assert_equal(dict1[key], dict2[key], msg='Key %s' % (key,))
             except AssertionError as err:
-                yield unic(err)
+                yield str(err)
+
+    def _validate_dictionary(self, dictionary, position=1):
+        if is_string(dictionary) or is_number(dictionary):
+            raise TypeError("Expected argument %d to be a dictionary or dictionary-like, "
+                            "got %s instead." % (position, type_name(dictionary)))
 
 
 class Collections(_List, _Dictionary):
-    """A test library providing keywords for handling lists and dictionaries.
+    """A library providing keywords for handling lists and dictionaries.
 
     ``Collections`` is Robot Framework's standard library that provides a
     set of keywords for handling Python lists and dictionaries. This
@@ -743,14 +835,18 @@ class Collections(_List, _Dictionary):
     From Dictionary`) and for verifying their contents (e.g. `Lists
     Should Be Equal`, `Dictionary Should Contain Value`).
 
+    == Table of contents ==
+
+    %TOC%
+
     = Related keywords in BuiltIn =
 
     Following keywords in the BuiltIn library can also be used with
     lists and dictionaries:
 
-    | = Keyword Name =             | = Applicable With = | = Comment = |
+    | = Keyword Name =             | = Applicable With = |
     | `Create List`                | lists |
-    | `Create Dictionary`          | dicts | Was in Collections until RF 2.9. |
+    | `Create Dictionary`          | dicts |
     | `Get Length`                 | both  |
     | `Length Should Be`           | both  |
     | `Should Be Empty`            | both  |
@@ -776,13 +872,12 @@ class Collections(_List, _Dictionary):
 
     Some keywords accept arguments that are handled as Boolean values true or
     false. If such an argument is given as a string, it is considered false if
-    it is either empty or case-insensitively equal to ``false`` or ``no``.
-    Keywords verifying something that allow dropping actual and expected values
-    from the possible error message also consider string ``no values`` as false.
-    Other strings are considered true regardless their value, and other
-    argument types are tested using same
-    [http://docs.python.org/2/library/stdtypes.html#truth-value-testing|rules
-    as in Python].
+    it is an empty string or equal to ``FALSE``, ``NONE``, ``NO``, ``OFF`` or
+    ``0``, case-insensitively. Keywords verifying something that allow dropping
+    actual and expected values from the possible error message also consider
+    string ``no values`` to be false. Other strings are considered true
+    regardless their value, and other argument types are tested using the same
+    [http://docs.python.org/library/stdtypes.html#truth|rules as in Python].
 
     True examples:
     | `Should Contain Match` | ${list} | ${pattern} | case_insensitive=True    | # Strings are generally true.    |
@@ -797,8 +892,7 @@ class Collections(_List, _Dictionary):
     | `Should Contain Match` | ${list} | ${pattern} | case_insensitive=${FALSE} | # Python ``False`` is false.   |
     | `Lists Should Be Equal` | ${x}   | ${y} | Custom error | values=no values | # ``no values`` works with ``values`` argument |
 
-    Note that prior to Robot Framework 2.9 some keywords considered all
-    non-empty strings, including ``False``, to be true.
+    Considering ``OFF`` and ``0`` false is new in Robot Framework 3.1.
 
     = Data in examples =
 
@@ -818,8 +912,6 @@ class Collections(_List, _Dictionary):
                              whitespace_insensitive=False):
         """Fails if ``pattern`` is not found in ``list``.
 
-        See `List Should Contain Value` for an explanation of ``msg``.
-
         By default, pattern matching is similar to matching files in a shell
         and is case-sensitive and whitespace-sensitive. In the pattern syntax,
         ``*`` matches to anything and ``?`` matches to any single character. You
@@ -828,7 +920,7 @@ class Collections(_List, _Dictionary):
 
         If you prepend ``regexp=`` to your pattern, your pattern will be used
         according to the Python
-        [http://docs.python.org/2/library/re.html|re module] regular expression
+        [http://docs.python.org/library/re.html|re module] regular expression
         syntax. Important note: Backslashes are an escape character, and must
         be escaped with another backslash (e.g. ``regexp=\\\\d{6}`` to search for
         ``\\d{6}``). See `BuiltIn.Should Match Regexp` for more details.
@@ -841,7 +933,7 @@ class Collections(_List, _Dictionary):
 
         Non-string values in lists are ignored when matching patterns.
 
-        The given list is never altered by this keyword.
+        Use the ``msg`` argument to override the default error message.
 
         See also ``Should Not Contain Match``.
 
@@ -852,9 +944,8 @@ class Collections(_List, _Dictionary):
         | Should Contain Match | ${list} | a*  | case_insensitive=True       | | # Match strings beginning with 'a' or 'A'. |
         | Should Contain Match | ${list} | ab* | whitespace_insensitive=yes  | | # Match strings beginning with 'ab' with possible whitespace ignored. |
         | Should Contain Match | ${list} | ab* | whitespace_insensitive=true | case_insensitive=true | # Same as the above but also ignore case. |
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         matches = _get_matches_in_iterable(list, pattern, case_insensitive,
                                            whitespace_insensitive)
         default = "%s does not contain match for pattern '%s'." \
@@ -868,9 +959,8 @@ class Collections(_List, _Dictionary):
 
         Exact opposite of `Should Contain Match` keyword. See that keyword
         for information about arguments and usage in general.
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         matches = _get_matches_in_iterable(list, pattern, case_insensitive,
                                            whitespace_insensitive)
         default = "%s contains match for pattern '%s'." \
@@ -888,9 +978,8 @@ class Collections(_List, _Dictionary):
         | ${matches}= | Get Matches | ${list} | a* | # ${matches} will contain any string beginning with 'a' |
         | ${matches}= | Get Matches | ${list} | regexp=a.* | # ${matches} will contain any string beginning with 'a' (regexp version) |
         | ${matches}= | Get Matches | ${list} | a* | case_insensitive=${True} | # ${matches} will contain any string beginning with 'a' or 'A' |
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         return _get_matches_in_iterable(list, pattern, case_insensitive,
                                         whitespace_insensitive)
 
@@ -905,9 +994,8 @@ class Collections(_List, _Dictionary):
         | ${count}= | Get Match Count | ${list} | a* | # ${count} will be the count of strings beginning with 'a' |
         | ${count}= | Get Match Count | ${list} | regexp=a.* | # ${matches} will be the count of strings beginning with 'a' (regexp version) |
         | ${count}= | Get Match Count | ${list} | a* | case_insensitive=${True} | # ${matches} will be the count of strings beginning with 'a' or 'A' |
-
-        New in Robot Framework 2.8.6.
         """
+        _List._validate_list(self, list)
         return len(self.get_matches(list, pattern, case_insensitive,
                                     whitespace_insensitive))
 
