@@ -1,4 +1,5 @@
-#  Copyright 2008-2015 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Networks
+#  Copyright 2016-     Robot Framework Foundation
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -14,17 +15,21 @@
 
 import copy
 import os.path
+import os
 
 from robot.output import LOGGER
-from robot.errors import FrameworkError
-from robot.utils import normpath, seq2str2, is_string
+from robot.errors import FrameworkError, DataError
+from robot.utils import normpath, seq2str, seq2str2, is_string
 
 from .builder import ResourceFileBuilder
 from .handlerstore import HandlerStore
 from .testlibraries import TestLibrary
 
 
-class Importer(object):
+RESOURCE_EXTENSIONS = ('.resource', '.robot', '.txt', '.tsv', '.rst', '.rest')
+
+
+class Importer:
 
     def __init__(self):
         self._library_cache = ImportCache()
@@ -33,6 +38,10 @@ class Importer(object):
     def reset(self):
         self.__init__()
 
+    def close_global_library_listeners(self):
+        for lib in self._library_cache.values():
+            lib.close_global_listeners()
+
     def import_library(self, name, args, alias, variables):
         lib = TestLibrary(name, args, variables, create_handlers=False)
         positional, named = lib.positional_args, lib.named_args
@@ -44,6 +53,7 @@ class Importer(object):
         return lib
 
     def import_resource(self, path):
+        self._validate_resource_extension(path)
         if path in self._resource_cache:
             LOGGER.info("Found resource file '%s' from cache" % path)
         else:
@@ -51,11 +61,18 @@ class Importer(object):
             self._resource_cache[path] = resource
         return self._resource_cache[path]
 
+    def _validate_resource_extension(self, path):
+        extension = os.path.splitext(path)[1]
+        if extension.lower() not in RESOURCE_EXTENSIONS:
+            raise DataError("Invalid resource file extension '%s'. "
+                            "Supported extensions are %s."
+                            % (extension, seq2str(RESOURCE_EXTENSIONS)))
+
     def _import_library(self, name, positional, named, lib):
-        args = positional + ['%s=%s' % arg for arg in sorted(named.items())]
+        args = positional + ['%s=%s' % arg for arg in named]
         key = (name, positional, named)
         if key in self._library_cache:
-            LOGGER.info("Found test library '%s' with arguments %s from cache"
+            LOGGER.info("Found library '%s' with arguments %s from cache."
                         % (name, seq2str2(args)))
             return self._library_cache[key]
         lib.create_handlers()
@@ -69,22 +86,19 @@ class Importer(object):
         LOGGER.info("Imported library '%s' with arguments %s "
                     "(version %s, %s type, %s scope, %d keywords%s)"
                     % (name, seq2str2(args), lib.version or '<unknown>',
-                       type, lib.scope.lower(), len(lib), listener))
-        if not lib and not lib.has_listener:
-            LOGGER.warn("Imported library '%s' contains no keywords" % name)
+                       type, lib.scope, len(lib), listener))
+        if not lib:
+            LOGGER.warn("Imported library '%s' contains no keywords." % name)
 
     def _copy_library(self, orig, name):
         # This is pretty ugly. Hopefully we can remove cache and copying
         # altogether in 3.0 and always just re-import libraries:
         # https://github.com/robotframework/robotframework/issues/2106
-        # Could then also remove __copy__ methods added to some handlers as
-        # a workaround for this IronPython bug:
-        # https://github.com/IronLanguages/main/issues/1192
         lib = copy.copy(orig)
         lib.name = name
-        lib.init_scope_handling()
-        lib.handlers = HandlerStore(orig.handlers.source,
-                                    orig.handlers.source_type)
+        lib.scope = type(lib.scope)(lib)
+        lib.reset_instance()
+        lib.handlers = HandlerStore(orig.handlers.source, orig.handlers.source_type)
         for handler in orig.handlers._normal.values():
             handler = copy.copy(handler)
             handler.library = lib
