@@ -16,9 +16,9 @@
 import re
 import sys
 
-# Version number typically updated by running `invoke set_version <version>`.
-# Run `invoke --help set_version` or see tasks.py for details.
-VERSION = '3.0.2'
+# Version number typically updated by running `invoke set-version <version>`.
+# Run `invoke --help set-version` or see tasks.py for details.
+VERSION = '5.0.1'
 
 
 def get_version(naked=False):
@@ -37,10 +37,6 @@ def get_full_version(program=None, naked=False):
 
 
 def get_interpreter():
-    if sys.platform.startswith('java'):
-        return 'Jython'
-    if sys.platform == 'cli':
-        return 'IronPython'
     if 'PyPy' in sys.version:
         return 'PyPy'
     return 'Python'
